@@ -80,12 +80,12 @@ class PackedFloatVector:
         indexPos: int = bitPos // 8
         bitPos %= 8
 
-        scale: float = 1.0 / self.m_Range
+        scale: float = (1.0 / self.m_Range) if self.m_Range else float("inf")
         if numChunks == -1:
             numChunks = self.m_NumItems // itemCountInChunk
-        end = chunkStride * numChunks / 4
+        end = int(chunkStride * numChunks / 4)
         data = []
-        for index in (0, end, chunkStride // 4):
+        for index in range(0, end, chunkStride // 4):
             for i in range(itemCountInChunk):
                 x = 0  # uint
                 bits = 0
@@ -101,7 +101,8 @@ class PackedFloatVector:
                         bitPos = 0
 
                 x &= uint((1 << self.m_BitSize) - 1)  # (uint)(1 << m_BitSize) - 1u
-                data.append(x / (scale * ((1 << self.m_BitSize) - 1)) + self.m_Start)
+                denomi = scale * ((1 << self.m_BitSize) - 1)
+                data.append((x / denomi if denomi else float("inf")) + self.m_Start)
 
         return data
 
@@ -128,23 +129,21 @@ class PackedIntVector:
         writer.align_stream()
 
     def UnpackInts(self):
-        data = []
+        data = [0] * self.m_NumItems
         indexPos = 0
         bitPos = 0
-        m_BitSize = self.m_BitSize
-
         for i in range(self.m_NumItems):
             bits = 0
-            entry = 0
-            while bits << m_BitSize:
-                entry |= (self.m_Data[indexPos] >> bitPos) << bits
-                num = min(m_BitSize - bits, 8 - bitPos)
+            data[i] = 0
+            while bits < self.m_BitSize:
+                data[i] |= (self.m_Data[indexPos] >> bitPos) << bits
+                num = min(self.m_BitSize - bits, 8 - bitPos)
                 bitPos += num
                 bits += num
-                if bitPos == 8:  #
+                if bitPos == 8:
                     indexPos += 1
                     bitPos = 0
-            data.append(entry & (1 << m_BitSize) - 1)
+            data[i] &= (1 << self.m_BitSize) - 1
         return data
 
 
@@ -157,7 +156,7 @@ class PackedQuatVector:
 
     def UnpackQuats(self):
         m_Data = self.m_Data
-        data = []
+        data = [None] * self.m_NumItems
         indexPos = 0
         bitPos = 0
 
@@ -341,7 +340,7 @@ class StreamedCurveKey:
         """
         # Stepped
         if self.coeff[0] == 0 and self.coeff[1] == 0 and self.coeff[2] == 0:
-            return float.PositiveInfinity
+            return float("inf")
 
         dx = max(dx, 0.0001)
         dy = rhs.value - self.value
@@ -365,7 +364,7 @@ class StreamedClip:
 
     def ReadData(self):
         frameList = []
-        buffer = self.data[0 : len(self.data) * 4]
+        buffer = b"".join(val.to_bytes(4, "big") for val in self.data)
         reader = EndianBinaryReader(buffer)
         while reader.Position < reader.Length:
             frameList.append(StreamedFrame(reader))
@@ -509,26 +508,26 @@ class AnimationClipBindingConstant:
         numMappings = reader.read_int()
         self.pptrCurveMapping = [PPtr(reader) for _ in range(numMappings)]  # Object
 
-        def FindBinding(self, index):
-            curves = 0
-            for b in self.genericBindings:
-                if b.typeID == ClassIDType.Transform:  #
-                    switch = b.attribute
+    def FindBinding(self, index):
+        curves = 0
+        for b in self.genericBindings:
+            if b.typeID == ClassIDType.Transform:  #
+                switch = b.attribute
 
-                    if switch in [1, 3, 4]:
-                        # case 1: #kBindTransformPosition
-                        # case 3: #kBindTransformScale
-                        # case 4: #kBindTransformEuler
-                        curves += 3
-                    elif switch == 2:  # kBindTransformRotation
-                        curves += 4
-                    else:
-                        curves += 1
+                if switch in [1, 3, 4]:
+                    # case 1: #kBindTransformPosition
+                    # case 3: #kBindTransformScale
+                    # case 4: #kBindTransformEuler
+                    curves += 3
+                elif switch == 2:  # kBindTransformRotation
+                    curves += 4
                 else:
                     curves += 1
-                if curves > index:
-                    return b
-            return None
+            else:
+                curves += 1
+            if curves > index:
+                return b
+        return None
 
 
 class AnimationType(IntEnum):
